//********************************************************************
//
//  nanorv32_regfile.v
//
//  This file is part of the NANORV32 project
//  http://
//
//  Description     :  Register file for NANORV32 processor
//
//
//  Author          :  rbarzic@gmail.com
//  Date            :  Mon Jan 10 22:19:54 2011
//
//********************************************************************
//
// Copyright (C) 2010 Ronan Barzic
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//********************************************************************


module nanorv32_regfile (/*AUTOARG*/
   // Outputs
   porta, portb,
			 
   // Inputs
   sel_porta, sel_portb, sel_rd, sel_rd2, rd, write_rd, rd2,
   write_rd2,hreadyd, 
   clk, rst_n,allow_hidden_use_of_x0
   );

   parameter NUM_REGS=32;
`include "nanorv32_parameters.v"

  // Selection
   input  [NANORV32_RF_PORTA_MSB:0] sel_porta;
   input [NANORV32_RF_PORTB_MSB:0]  sel_portb;
   input [NANORV32_RF_PORTRD_MSB:0] sel_rd;
   input [NANORV32_RF_PORTRD_MSB:0] sel_rd2;
   // Data ports
   output [NANORV32_DATA_MSB:0]     porta;
   output [NANORV32_DATA_MSB:0]     portb;
   input [NANORV32_DATA_MSB:0]      rd;
   input                            write_rd;
   input [NANORV32_DATA_MSB:0]      rd2;
   input                            write_rd2;
   input                            hreadyd;

   // only when using micro-rom code
   // we use x0 as a temporary register
   input                            allow_hidden_use_of_x0;


   input                            clk;
   input                            rst_n;

   /*AUTOINPUT*/

  /*AUTOOUTPUT*/

  /*AUTOREG*/
  // Beginning of automatic regs (for this module's undeclared outputs)

   wire [NANORV32_DATA_MSB:0] 	    sel_porta_oh;
   wire [NANORV32_DATA_MSB:0] 	    sel_portb_oh;
   wire [NANORV32_DATA_MSB:0] 	    sel_rd_oh;
   wire [NANORV32_DATA_MSB:0] 	    sel_rd2_oh;

   
   
   assign sel_porta_oh = (sel_porta == 0) ? 1 : (1'b1 << sel_porta);
   assign sel_portb_oh = (sel_portb == 0) ? 1 : (1'b1 << sel_portb);
   assign sel_rd_oh  = (sel_rd == 0) ? 1 : (1'b1 << sel_rd);
   assign sel_rd2_oh = (sel_rd2 == 0) ? 1 : (1'b1 << sel_rd2);				     
				     

   


nanorv32_regfile_core U_REG_FILE_CORE(/*AUTOINST*/
					      //Selection
					      .sel_porta(sel_porta_oh),
					      .sel_portb(sel_portb_oh),
					      .sel_rd(sel_rd_oh),
					      .sel_rd2(sel_rd2_oh),
					      //Data ports
					      .porta(porta),
					      .portb(portb),
					      .rd(rd),
					      .write_rd(write_rd),
					      .rd2(rd2),
					      .write_rd2(write_rd2),
					      .clk(clk),
					      .rst_n(rst_n),
				              .allow_hidden_use_of_x0(allow_hidden_use_of_x0)
				         
					      

					      
);




   


endmodule // nanorv32_regfile
